@@ -51,7 +51,8 @@ typedef enum
 {
 	EPHY_EMBED_SHELL_MODE_BROWSER,
 	EPHY_EMBED_SHELL_MODE_PRIVATE,
-	EPHY_EMBED_SHELL_MODE_APPLICATION
+	EPHY_EMBED_SHELL_MODE_APPLICATION,
+	EPHY_EMBED_SHELL_MODE_TEST
 } EphyEmbedShellMode;
 
 struct _EphyEmbedShell
@@ -79,10 +80,6 @@ GType		   ephy_embed_shell_get_type		(void);
 
 EphyEmbedShell	  *ephy_embed_shell_get_default		(void);
 
-GObject		  *ephy_embed_shell_get_favicon_cache	(EphyEmbedShell *shell);
-
-GObject		  *ephy_embed_shell_get_global_history	(EphyEmbedShell *shell);
-
 GObject		  *ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell);
 
 GObject		  *ephy_embed_shell_get_encodings	(EphyEmbedShell *shell);
