@@ -551,9 +551,7 @@ ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
   ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_OPEN);
 }
 
-#ifdef HAVE_WEBKIT2
-/* TODO: Downloads */
-#else
+#ifndef HAVE_WEBKIT2
 static gboolean
 download_requested_cb (WebKitWebView *web_view,
                        WebKitDownload *download,
@@ -595,7 +593,9 @@ ephy_embed_set_fullscreen_message (EphyEmbed *embed,
 
 static gboolean
 entering_fullscreen_cb (WebKitWebView *web_view,
+#ifndef HAVE_WEBKIT2
                         GObject *element,
+#endif
                         EphyEmbed *embed)
 {
   ephy_embed_set_fullscreen_message (embed, TRUE);
@@ -604,7 +604,9 @@ entering_fullscreen_cb (WebKitWebView *web_view,
 
 static gboolean
 leaving_fullscreen_cb (WebKitWebView *web_view,
+#ifndef HAVE_WEBKIT2
                        GObject *element,
+#endif
                        EphyEmbed *embed)
 {
   ephy_embed_set_fullscreen_message (embed, FALSE);
@@ -835,9 +837,11 @@ ephy_embed_constructed (GObject *object)
 
 #ifdef HAVE_WEBKIT2
   /* TODO: WebKitWebResource::send-request, Downloads */
-  g_signal_connect (web_view, "load-changed",
-                    G_CALLBACK (load_changed_cb),
-                    embed);
+  g_object_connect (web_view,
+                    "signal::load-changed", G_CALLBACK (load_changed_cb), embed,
+                    "signal::enter-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
+                    "signal::leave-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
+                    NULL);
 #else
   g_object_connect (web_view,
                     "signal::notify::load-status", G_CALLBACK (load_status_changed_cb), embed,
