@@ -59,6 +59,9 @@ void         ephy_embed_remove_top_widget        (EphyEmbed  *embed,
                                                   GtkWidget  *widget);
 void         ephy_embed_auto_download_url        (EphyEmbed  *embed,
                                                   const char *url);
+void         ephy_embed_entering_fullscreen      (EphyEmbed *embed);
+void         ephy_embed_leaving_fullscreen       (EphyEmbed *embed);
+
 G_END_DECLS
 
 #endif
