@@ -119,9 +119,16 @@ ephy_file_download_dir (void)
 /**
  * ephy_file_get_downloads_dir:
  *
- * Gets the full path to the downloads dir. This uses ephy_file_downloads_dir()
- * internally and hence is locale dependant. Note that this can return %NULL if
- * not even the user homedir path can be found.
+ * Returns a proper downloads destination by checking the
+ * EPHY_PREFS_STATE_DOWNLOAD_DIR GSettings key and following this logic:
+ *
+ *  - An absolute path: considered user-set, use this value directly.
+ *
+ *  - "Desktop" keyword in GSettings: the directory returned by
+ *    ephy_file_desktop_dir().
+ *
+ *  - "Downloads" keyword in GSettings, or any other value: the XDG
+ *  downloads directory, or ~/Downloads.
  *
  * Returns: a newly-allocated string containing the path to the downloads dir.
  **/
@@ -133,11 +140,11 @@ ephy_file_get_downloads_dir (void)
 	download_dir = g_settings_get_string (EPHY_SETTINGS_STATE,
 					      EPHY_PREFS_STATE_DOWNLOAD_DIR);
 
-	/* Emergency download destination */
-	if (g_str_equal (download_dir, "Downloads"))
-		download_dir = ephy_file_download_dir ();
-	else if (g_str_equal (download_dir, "Desktop") || g_path_is_absolute (download_dir) != TRUE)
+	if (g_str_equal (download_dir, "Desktop"))
 		download_dir = ephy_file_desktop_dir ();
+	if (g_str_equal (download_dir, "Downloads") ||
+	    g_path_is_absolute (download_dir) != TRUE)
+		download_dir = ephy_file_download_dir ();
 
 	return download_dir;
 }
@@ -272,22 +279,21 @@ ephy_dot_dir (void)
 /**
  * ephy_file_helpers_init:
  * @profile_dir: directory to use as Epiphany's profile
- * @private_profile: %TRUE if we should use a private profile
- * @keep_temp_dir: %TRUE to omit deleting the temp dir on exit
+ * @flags: the %EphyFileHelpersFlags for this session
  * @error: an optional #GError
  *
  * Initializes Epiphany file helper functions, sets @profile_dir as Epiphany's
  * profile dir and whether the running session will be private.
  *
- * Returns: %FALSE if the profile dir couldn't be created or accesed
+ * Returns: %FALSE if the profile dir couldn't be created or accessed
  **/
 gboolean
 ephy_file_helpers_init (const char *profile_dir,
-			gboolean private_profile,
-			gboolean keep_temp_dir,
+			EphyFileHelpersFlags flags,
 			GError **error)
 {
 	const char *uuid;
+	gboolean private_profile;
 
 	/* See if we've been calling ourself, and abort if we have */
 	uuid = g_getenv (EPHY_UUID_ENVVAR);
@@ -307,7 +313,8 @@ ephy_file_helpers_init (const char *profile_dir,
 				       (GDestroyNotify) g_free,
 				       (GDestroyNotify) g_free);
 
-	keep_temp_directory = keep_temp_dir;
+	keep_temp_directory = flags & EPHY_FILE_HELPERS_KEEP_TEMP_DIR;
+	private_profile = flags & EPHY_FILE_HELPERS_PRIVATE_PROFILE;
 
 	if (private_profile && profile_dir != NULL)
 	{
@@ -331,13 +338,15 @@ ephy_file_helpers_init (const char *profile_dir,
 	}
 	else
 	{
-		dot_dir = g_build_filename (g_get_home_dir (),
-					    GNOME_DOT_GNOME,
+		dot_dir = g_build_filename (g_get_user_config_dir (),
 					    "epiphany",
 					    NULL);
 	}
-	
-	return ephy_ensure_dir_exists (ephy_dot_dir (), error);
+
+	if (flags & EPHY_FILE_HELPERS_ENSURE_EXISTS)
+		return ephy_ensure_dir_exists (ephy_dot_dir (), error);
+	else
+		return TRUE;
 }
 
 static void
@@ -962,39 +971,53 @@ ephy_file_delete_uri (const char *uri)
 	g_object_unref (file);
 }
 
-static gchar *
-get_accels_filename (void)
+/**
+ * ephy_file_create_data_uri_for_filename:
+ * @filename: the filename of a local path
+ * @mime_type: the MIME type of the filename, or %NULL
+ *
+ * Create a data uri using the contents of @filename.
+ * If @mime_type is %NULL, the %G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
+ * attribute of @filename will be used.
+ *
+ * Returns: a new allocated string containg the data uri, or %NULL if the
+ *   data uri could not be created
+ */
+char *ephy_file_create_data_uri_for_filename (const char *filename,
+					      const char *mime_type)
 {
-        const char *home;
+	gchar *data;
+	gsize data_length;
+	gchar *base64;
+	gchar *uri = NULL;
+	GFileInfo *file_info = NULL;
 
-        home = g_get_home_dir();
-        if (!home)
-                return NULL;
-        return g_build_filename (home, GNOME_DOT_GNOME, "accels", PACKAGE, NULL);
-}
-
-void
-ephy_file_load_accels (void)
-{
-        char *filename;
-
-        filename = get_accels_filename ();
-        if (!filename)
-                return;
-
-        gtk_accel_map_load (filename);
-        g_free (filename);
-}
-
-void
-ephy_file_save_accels (void)
-{
-        char *filename;
-
-        filename = get_accels_filename ();
-        if (!filename)
-                return;
-
-        gtk_accel_map_save (filename);
-        g_free (filename);
+	g_return_val_if_fail (filename != NULL, NULL);
+
+	if (!g_file_get_contents (filename, &data, &data_length, NULL))
+		return NULL;
+
+	base64 = g_base64_encode ((const guchar *)data, data_length);
+	g_free (data);
+
+	if (!mime_type) {
+		GFile *file;
+
+		file = g_file_new_for_path (filename);
+		file_info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+					       G_FILE_QUERY_INFO_NONE, NULL, NULL);
+		if (file_info)
+			mime_type = g_file_info_get_content_type (file_info);
+
+		g_object_unref (file);
+	}
+
+	if (mime_type)
+		uri = g_strdup_printf ("data:%s;charset=utf8;base64,%s", mime_type, base64);
+	g_free(base64);
+
+	if (file_info)
+		g_object_unref (file_info);
+
+	return uri;
 }
