@@ -269,7 +269,7 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
   if (error) {
     g_object_unref (priv->history_database);
     priv->history_database = NULL;
-    g_error ("Could not open history database: %s", error->message);
+    g_error ("Could not open history database at %s: %s", priv->history_filename, error->message);
     g_error_free (error);
     return FALSE;
   }
@@ -382,6 +382,7 @@ run_history_service_thread (EphyHistoryService *self)
   } while (!ephy_history_service_is_scheduled_to_quit (self));
 
   ephy_history_service_close_database_connections (self);
+  ephy_history_service_execute_quit (self, NULL, NULL);
 
   return NULL;
 }
