@@ -28,7 +28,7 @@
 #include "ephy-dnd.h"
 #include "ephy-debug.h"
 #include "ephy-gui.h"
-#include "ephy-request-about.h"
+#include "ephy-about-handler.h"
 
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -78,6 +78,7 @@ struct _EphyLocationEntryPrivate
 	guint apply_colors : 1;
 	guint needs_reset : 1;
 	guint show_lock : 1;
+	guint show_favicon : 1;
 };
 
 static const GtkTargetEntry url_drag_types [] =
@@ -102,6 +103,16 @@ static void extracell_data_func (GtkCellLayout *cell_layout,
 				 GtkTreeIter *iter,
 				 gpointer data);
 
+enum
+{
+	PROP_0,
+	PROP_LOCATION,
+	PROP_FAVICON,
+	PROP_LOCK_STOCK,
+	PROP_SHOW_LOCK,
+	PROP_SHOW_FAVICON
+};
+
 enum signalsEnum
 {
 	USER_CHANGED,
@@ -114,6 +125,59 @@ static gint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_ENTRY)
 
+static void
+ephy_location_entry_set_property (GObject *object,
+				  guint prop_id,
+				  const GValue *value,
+				  GParamSpec *pspec)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
+
+	switch (prop_id)
+	{
+	case PROP_LOCATION:
+		ephy_location_entry_set_location (entry,
+						  g_value_get_string (value));
+		break;
+	case PROP_FAVICON:
+		ephy_location_entry_set_favicon (entry,
+						 g_value_get_object (value));
+		break;
+	case PROP_LOCK_STOCK:
+		ephy_location_entry_set_lock_stock (entry,
+						    g_value_get_string (value));
+		break;
+	case PROP_SHOW_LOCK:
+		ephy_location_entry_set_show_lock (entry,
+						   g_value_get_boolean (value));
+		break;
+	case PROP_SHOW_FAVICON:
+		ephy_location_entry_set_show_favicon (entry,
+						      g_value_get_boolean (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id,pspec);
+	}
+}
+
+static void
+ephy_location_entry_get_property (GObject *object,
+				  guint prop_id,
+				  GValue *value,
+				  GParamSpec *pspec)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
+
+	switch (prop_id)
+	{
+	case PROP_LOCATION:
+		g_value_set_string (value, ephy_location_entry_get_location (entry));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id,pspec);
+	}
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -140,8 +204,70 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->get_property = ephy_location_entry_get_property;
+	object_class->set_property = ephy_location_entry_set_property;
 	object_class->finalize = ephy_location_entry_finalize;
 
+	/**
+	* EphyLocationEntry:location:
+	*
+	* The current location.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_LOCATION,
+					 g_param_spec_string ("location",
+							      "Location",
+							      "The current location",
+							      "",
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:favicon:
+	*
+	* The icon corresponding to the current location.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_FAVICON,
+					 g_param_spec_object ("favicon",
+							      "Favicon",
+							      "The icon corresponding to the current location",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:lock-stock-id:
+	*
+	* Stock id of the security icon.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_STOCK,
+					 g_param_spec_string  ("lock-stock-id",
+							       "Lock Stock ID",
+							       "Stock id of the security icon",
+							       NULL,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:show-lock:
+	*
+	* If we should show the security icon.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_LOCK,
+					 g_param_spec_boolean ("show-lock",
+							       "Show Lock",
+							       "If we should show the security icon",
+							       FALSE,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_FAVICON,
+					 g_param_spec_boolean ("show-favicon",
+							       "Show Favicon",
+							       "Whether to show the favicon",
+							       TRUE,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
        /**
 	* EphyLocationEntry::user-changed:
 	* @entry: the object on which the signal is emitted
@@ -240,13 +366,13 @@ update_favicon (EphyLocationEntry *lentry)
 	/* Only show the favicon if the entry's text is the
 	 * address of the current page.
 	 */
-	if (priv->favicon != NULL && priv->original_address)
+	if (priv->show_favicon && priv->favicon != NULL && priv->original_address)
 	{
 		gtk_entry_set_icon_from_pixbuf (entry,
 						GTK_ENTRY_ICON_PRIMARY,
 						priv->favicon);
 	}
-	else
+	else if (priv->show_favicon)
 	{
 		/* Here we could consider using fallback favicon that matches
 		 * the page MIME type, though text/html should be good enough
@@ -256,6 +382,12 @@ update_favicon (EphyLocationEntry *lentry)
 						   GTK_ENTRY_ICON_PRIMARY,
 						   "text-html");
 	}
+	else
+	{
+		gtk_entry_set_icon_from_icon_name (entry,
+						   GTK_ENTRY_ICON_PRIMARY,
+						   NULL);
+	}
 }
 
 static void
@@ -780,6 +912,7 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->block_update = FALSE;
 	p->saved_text = NULL;
 	p->show_lock = FALSE;
+	p->show_favicon = TRUE;
 	p->dns_prefetch_handler = 0;
 
 	ephy_location_entry_construct_contents (le);
@@ -807,9 +940,13 @@ free_prefetch_helper (PrefetchHelper *helper)
 static gboolean
 do_dns_prefetch (PrefetchHelper *helper)
 {
+#ifdef HAVE_WEBKIT2
+	/* TODO: Network Features */
+#else
 	SoupSession *session = webkit_get_default_session ();
 
 	soup_session_prepare_for_uri (session, helper->uri);
+#endif
 
 	helper->entry->priv->dns_prefetch_handler = 0;
 
@@ -1371,6 +1508,21 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+void
+ephy_location_entry_set_show_favicon (EphyLocationEntry *entry,
+				      gboolean show_favicon)
+{
+	EphyLocationEntryPrivate *priv;
+
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
+
+	priv = entry->priv;
+
+	priv->show_favicon = show_favicon != FALSE;
+
+	update_favicon (entry);
+}
+
 /**
  * ephy_location_entry_set_show_lock:
  * @entry: an #EphyLocationEntry widget
