@@ -1139,9 +1139,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		key_normal = (priority == EPHY_NODE_NORMAL_PRIORITY);
 
 		EPHY_TOPIC_ACTION_NAME_PRINTF (name, node);
-
-		g_list_free (selected);
 	}
+	g_list_free (selected);
 
 	selected = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->bm_view));
 	if (bmk_focus && selected)
@@ -1154,9 +1153,8 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 		mutable = !ephy_node_get_property_boolean (node, EPHY_NODE_BMK_PROP_IMMUTABLE);
 		
 		EPHY_BOOKMARK_ACTION_NAME_PRINTF (name, node);
-
-		g_list_free (selected);
 	}
+	g_list_free (selected);
 
 	open_in_window_label = ngettext ("Open in New _Window",
 					 "Open in New _Windows",
@@ -1460,6 +1458,22 @@ node_dropped_cb (EphyNodeView *view,
 	}
 }
 
+#ifdef HAVE_WEBKIT2
+/* TODO: Favicons */
+#else
+static gboolean
+webkit_favicon_database_has_favicon (WebKitFaviconDatabase *database, const char *page_uri)
+{
+    gboolean result;
+    char *uri;
+
+    uri = webkit_favicon_database_get_favicon_uri (database, page_uri);
+    result = (uri != NULL);
+    g_free (uri);
+
+    return result;
+}
+
 static void
 icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GValue *value)
 {
@@ -1469,10 +1483,14 @@ icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GValue *v
 
     g_value_take_object (value, favicon);
 }
+#endif
 
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
+#ifdef HAVE_WEBKIT2
+        /* TODO: Favicons */
+#else
 	const char *page_location;
 	GdkPixbuf *favicon = NULL;
 
@@ -1489,7 +1507,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
                 favicon = webkit_favicon_database_try_get_favicon_pixbuf (database, page_location,
 									  FAVICON_SIZE, FAVICON_SIZE);
 
-		if (!favicon && webkit_favicon_database_get_favicon_uri (database, page_location))
+		if (!favicon && webkit_favicon_database_has_favicon (database, page_location))
 		  webkit_favicon_database_get_favicon_pixbuf (database, page_location,
 							      FAVICON_SIZE, FAVICON_SIZE, NULL,
 							      (GAsyncReadyCallback) icon_loaded_cb, value);
@@ -1497,6 +1515,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
 	g_value_take_object (value, favicon);
+#endif
 }
 
 static void
