@@ -56,6 +56,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <libnotify/notify.h>
+#include <libsoup/soup.h>
 #include <string.h>
 #ifdef HAVE_WEBKIT2
 #include <webkit2/webkit2.h>
@@ -279,20 +280,26 @@ window_cmd_file_open (GtkAction *action,
 static char *
 get_suggested_filename (EphyWebView *view)
 {
-#ifdef HAVE_WEBKIT2
-	/* TODO: Resources */
-	return g_strdup ("WebPage");
-#else
 	char *suggested_filename;
 	const char *mimetype;
+#ifdef HAVE_WEBKIT2
+	WebKitURIResponse *response;
+#else
 	WebKitWebFrame *frame;
 	WebKitWebDataSource *data_source;
+#endif
 	WebKitWebResource *web_resource;
 
+#ifdef HAVE_WEBKIT2
+	web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
+	response = webkit_web_resource_get_response (web_resource);
+	mimetype = webkit_uri_response_get_mime_type (response);
+#else
 	frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view));
 	data_source = webkit_web_frame_get_data_source (frame);
 	web_resource = webkit_web_data_source_get_main_resource (data_source);
 	mimetype = webkit_web_resource_get_mime_type (web_resource);
+#endif
 
 	if ((g_ascii_strncasecmp (mimetype, "text/html", 9)) == 0)
 	{
@@ -307,7 +314,6 @@ get_suggested_filename (EphyWebView *view)
 	}
 
 	return suggested_filename;
-#endif
 }
 
 void
@@ -374,7 +380,26 @@ take_page_snapshot_and_set_image (EphyApplicationDialogData *data)
 }
 
 #ifdef HAVE_WEBKIT2
-/* TODO: Downloads */
+static void
+download_finished_cb (WebKitDownload *download,
+		      EphyApplicationDialogData *data)
+{
+	char *filename;
+
+	filename = g_filename_from_uri (webkit_download_get_destination (download), NULL, NULL);
+	gtk_image_set_from_file (GTK_IMAGE (data->image), filename);
+	g_free (filename);
+}
+
+static void
+download_failed_cb (WebKitDownload *download,
+		    GError *error,
+		    EphyApplicationDialogData *data)
+{
+	g_signal_handlers_disconnect_by_func (download, download_finished_cb, data);
+	/* Something happened, default to a page snapshot. */
+	take_page_snapshot_and_set_image (data);
+}
 #else
 static void
 download_status_changed_cb (WebKitDownload *download,
@@ -406,25 +431,39 @@ download_status_changed_cb (WebKitDownload *download,
 static void
 download_icon_and_set_image (EphyApplicationDialogData *data)
 {
-#ifdef HAVE_WEBKIT2
-	/* TODO: Downloads */
-#else
+#ifndef HAVE_WEBKIT2
 	WebKitNetworkRequest *request;
+#endif
 	WebKitDownload *download;
 	char *destination, *destination_uri, *tmp_filename;
 
+#ifdef HAVE_WEBKIT2
+	download = webkit_web_context_download_uri (webkit_web_context_get_default (),
+						    data->icon_href);
+#else
 	request = webkit_network_request_new (data->icon_href);
 	download = webkit_download_new (request);
 	g_object_unref (request);
+#endif
 
 	tmp_filename = ephy_file_tmp_filename ("ephy-download-XXXXXX", NULL);
 	destination = g_build_filename (ephy_file_tmp_dir (), tmp_filename, NULL);
 	destination_uri = g_filename_to_uri (destination, NULL, NULL);
+#ifdef HAVE_WEBKIT2
+	webkit_download_set_destination (download, destination_uri);
+#else
 	webkit_download_set_destination_uri (download, destination_uri);
+#endif
 	g_free (destination);
 	g_free (destination_uri);
 	g_free (tmp_filename);
 
+#ifdef HAVE_WEBKIT2
+	g_signal_connect (download, "finished",
+			  G_CALLBACK (download_finished_cb), data);
+	g_signal_connect (download, "failed",
+			  G_CALLBACK (download_failed_cb), data);
+#else
 	g_signal_connect (download, "notify::status",
 			  G_CALLBACK (download_status_changed_cb), data);
 
@@ -1007,16 +1046,46 @@ save_temp_source_write_cb (GOutputStream *ostream, GAsyncResult *result, GString
 }
 
 #ifdef HAVE_WEBKIT2
-/* TODO: Resources */
-#else
+static void
+get_main_resource_data_cb (WebKitWebResource *resource, GAsyncResult *result, GOutputStream *ostream)
+{
+	guchar *data;
+	gsize data_length;
+	GString *data_str;
+	GError *error = NULL;
+
+	data = webkit_web_resource_get_data_finish (resource, result, &data_length, &error);
+	if (error) {
+		g_warning ("Unable to get main resource data: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	/* We create a new GString here because we need to make sure
+	 * we keep writing in case of partial writes */
+	data_str = g_string_new_len ((gchar *)data, data_length);
+	g_free (data);
+
+	g_output_stream_write_async (ostream,
+				     data_str->str, data_str->len,
+				     G_PRIORITY_DEFAULT, NULL,
+				     (GAsyncReadyCallback)save_temp_source_write_cb,
+				     data_str);
+}
+#endif
+
 static void
 save_temp_source_replace_cb (GFile *file, GAsyncResult *result, EphyEmbed *embed)
 {
 	EphyWebView *view;
+#ifdef HAVE_WEBKIT2
+	WebKitWebResource *resource;
+#else
 	WebKitWebFrame *frame;
 	WebKitWebDataSource *data_source;
 	GString *const_data;
 	GString *data;
+#endif
 	GFileOutputStream *ostream;
 	GError *error = NULL;
 
@@ -1045,6 +1114,12 @@ save_temp_source_replace_cb (GFile *file, GAsyncResult *result, EphyEmbed *embed
 				g_object_ref (embed),
 				g_object_unref);
 
+#ifdef HAVE_WEBKIT2
+	resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
+	webkit_web_resource_get_data (resource, NULL,
+				      (GAsyncReadyCallback)get_main_resource_data_cb,
+				      ostream);
+#else
 	frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view));
 	data_source = webkit_web_frame_get_data_source (frame);
 	const_data = webkit_web_data_source_get_data (data_source);
@@ -1061,8 +1136,8 @@ save_temp_source_replace_cb (GFile *file, GAsyncResult *result, EphyEmbed *embed
 				     G_PRIORITY_DEFAULT, NULL,
 				     (GAsyncReadyCallback)save_temp_source_write_cb,
 				     data);
-}
 #endif
+}
 
 static void
 save_temp_source (EphyEmbed *embed,
@@ -1087,15 +1162,11 @@ save_temp_source (EphyEmbed *embed,
 	}
 
 	file = g_file_new_for_path (tmp);
-#ifdef HAVE_WEBKIT2
-	/* TODO: Resources */
-#else
 	g_file_replace_async (file, NULL, FALSE,
 			      G_FILE_CREATE_REPLACE_DESTINATION|G_FILE_CREATE_PRIVATE,
 			      G_PRIORITY_DEFAULT, NULL,
 			      (GAsyncReadyCallback)save_temp_source_replace_cb,
 			      embed);
-#endif
 
 	g_object_unref (file);
 	g_free (tmp);
@@ -1115,12 +1186,16 @@ window_cmd_view_page_source (GtkAction *action,
 
 	address = ephy_web_view_get_address (ephy_embed_get_web_view (embed));
 
+#ifdef HAVE_WEBKIT2
+	/* TODO: View Source */
+#else
 	if (g_settings_get_boolean (EPHY_SETTINGS_MAIN,
 				    EPHY_PREFS_INTERNAL_VIEW_SOURCE))
 	{
 		view_source_embedded (address, embed);
 		return;
 	}
+#endif
 
 	user_time = gtk_get_current_event_time ();
 
@@ -1159,7 +1234,7 @@ window_cmd_help_about (GtkAction *action,
 		   "51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA")
 	};
 
-	char *licence, *comments;
+	char *licence = NULL, *comments = NULL;
 	GKeyFile *key_file;
 	GError *error = NULL;
 	char **list, **authors, **contributors, **past_authors, **artists, **documenters;
@@ -1237,7 +1312,11 @@ window_cmd_help_about (GtkAction *action,
 	g_key_file_free (key_file);
 
 #ifdef HAVE_WEBKIT2
-	/* TODO: WebKit Version */
+	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
+				      "Powered by WebKit %d.%d.%d"),
+				    webkit_get_major_version (),
+				    webkit_get_minor_version (),
+				    webkit_get_micro_version ());
 #else
 	comments = g_strdup_printf (_("Lets you view web pages and find information on the internet.\n"
 	                              "Powered by WebKit %d.%d.%d"),
